@@ -21,6 +21,7 @@
 
 # hsh built in commands.
 
+import errno
 import logging
 import os
 import os.path
@@ -30,17 +31,23 @@ import hsh
 import hsh.aliases
 import hsh.exceptions
 import hsh.config
+import hsh.system
 import welcome
 import jobs
 
 class bi_cd(jobs.BuiltInJob):
     def do_start(self):
-        if len(self.cmd) == 1:
-            os.chdir(os.path.expanduser('~'))
-        elif len(self.cmd) == 2:
-            os.chdir(self.cmd[1])
-        else:
-            hsh.display_obj.show_alert("cd takes at most one parameter")
+        try:
+            if len(self.cmd) == 1:
+                hsh.system.set_cwd(os.path.expanduser('~'))
+            elif len(self.cmd) == 2:
+                hsh.system.set_cwd(self.cmd[1])
+            else:
+                hsh.display_obj.show_alert("cd takes at most one parameter")
+        except OSError, ose:
+            self.state = "Failed"
+            self.output[:] = [os.strerror(ose.errno)]
+            hsh.display_obj.show_alert(os.strerror(ose.errno))
 
 jobs.register_builtin("cd", bi_cd)
 
@@ -131,7 +138,14 @@ class bi_env(jobs.BuiltInJob):
         if len(self.cmd) == 1:
             hsh.display_obj.show_view("Environment")
         elif len(self.cmd) == 2:
-            del os.environ[self.cmd[1]]
+            try:
+                del os.environ[self.cmd[1]]
+            except KeyError, ke:
+                self.state = "Failed"
+                msg = "Variable not found (%s)" % str(ke)
+                self.output[:] = [msg]
+                hsh.display_obj.show_alert(msg)
+                return
             hsh.display_obj.get_view("Environment").set_dirty()
         elif len(self.cmd) == 3:
             os.environ[self.cmd[1]] = self.cmd[2]
