@@ -43,18 +43,29 @@ class ParseException(Exception):
 # Static functions.
 
 # re which matches strings that require glob expansion.
-glob_match = re.compile(r"(^|[^\\])([\*\?\[\]])")
-# re which matches characters to escape to prevent glob expansion, including \.
-glob_chars = re.compile(r"([\*\?\[\]\\])")
+glob_match = re.compile(r"(^|[^\\])(\\\\)*([*?\[\]])")
+# re's to match characters to escape to prevent glob expansion, including \.
+glob_chars = re.compile(r"([*?\[\]])")
+glob_chars_slash = re.compile(r"([*?\[\]\\])")
 # re to unescape everything except escaped glob symbols and \.
-unescaper_nonglobs = re.compile(r"\\([^\*\?\[\]\\])")
+
+# Any slash which matches outside the group '\\??' is removed, and everything
+# else is preserved.
+
+# It matches every character in the string
+unescaper_nonglobs = re.compile(r"""(?x)    # Verbose - ignore whitespace
+   \\??            # The backslash which will be removed.
+   (  [^*?\[\]\\]  # Preserve non-glob char, consuming optional preceding \ 
+    | \\[*?\[\]\\] # Preserve any escaped glob characters
+    |$)""")        # Match so trailing \ is removed.
+
 # re to unescape a string of all backslash escapes.
 unescaper = re.compile(r"\\(.)")
 
 def escape_glob_chars(text):
     """Given a string, return a version of it with the special glob characters
     escaped.  Backslashes are also escaped to prevent ambiguity."""
-    return glob_chars.sub(r"\\\1", text)
+    return glob_chars_slash.sub(r"\\\1", text)
 
 whitespace = re.compile(r"\s")
 def format_cmd(args):
@@ -107,11 +118,11 @@ class Token(object):
         the end, and txt to the empty string.
 
         Returns a 3-tuple:
-          1] string: text which wasn't used by this token and spills over to
+          1. string: text which wasn't used by this token and spills over to
              the next token.
-          2] boolean: True means that this token may absorb some characters
+          2. boolean: True means that this token may absorb some characters
              from the next token.  Spill over is not possible in this case.
-          3] boolean: True means that this token needs to be deleted.  May
+          3. boolean: True means that this token needs to be deleted.  May
              include spillover.
 
         The default implementation checks the insertion range, raising an
@@ -413,13 +424,13 @@ class LeafToken(Token):
     def dump(self):
         return ["%s %s\n" % (str(self), self.__class__.__name__)]
 
-class CommandText(AggregateToken):
+class CommandToken(AggregateToken):
     # The root of the token tree, contains a list of Args and Arg separators.
 
     def __init__(self, do_aliases=True):
-        """Create a new CommandText token.  If aliases is False, do not expand
+        """Create a new CommandToken token.  If aliases is False, do not expand
         aliases in this command, to prevent unbounded recursion."""
-        super(CommandText, self).__init__()
+        super(CommandToken, self).__init__()
         self.do_aliases = do_aliases
 
     def insert(self, loc, txt):
@@ -435,7 +446,7 @@ class CommandText(AggregateToken):
     def modify(self, txt="", begin=0, end=None):
         # Handle the one case where super.modify() may leave an Arg at the
         # beginning instead of a Prog, and reparse the whole thing.
-        ret = super(CommandText, self).modify(txt, begin, end)
+        ret = super(CommandToken, self).modify(txt, begin, end)
         if len(self.tokens) > 0 and isinstance(self.tokens[0], Arg):
             rawtext = self.__str__()
             self.tokens = []
@@ -480,7 +491,7 @@ class CommandText(AggregateToken):
         res = []
         for tok in self.tokens:
             res.extend(tok.expand_directives())
-        return HashDirective.canonify(res)
+        return jobs.canonify_directives(res)
 
 class ArgOrProg(AggregateToken):
     # Intermediate class for implementation shared by Arg and Prog classes.
@@ -526,6 +537,7 @@ class Arg(ArgOrProg):
                 # Do glob expansion.  Unfortunately the results are ambiguous
                 # if an escaped glob character was in the original pattern or
                 # in the results.
+                unglobbed = unescaper.sub("\\1", unglobbed)
                 globbed = glob.glob(unglobbed)
                 globbed.sort()
                 globbedexp.extend(globbed)
@@ -545,10 +557,11 @@ class Arg(ArgOrProg):
         text = self.__str__()
         if len(expansions) > 1 or (len(expansions) == 0 and len(text) > 0):
             return []
-        # Use globbing to get completions, but escape globbing symbols first.
-        # This implementation of glob doesn't handle escapes properly, a
-        # drawback that needs to be rectified, and makes this code broken.
-        root = escape_glob_chars(expansions[0])
+        # Python's glob functionality doesn't provide a way to match globbing
+        # symbols literally, preventing correct completions on text with
+        # literal glob symbols.  As an inadequate work around, literal globbing
+        # symbols are replaced with ?, which will be ok on literal *.
+        root = glob_chars.sub(r"?", expansions[0])
         comps = glob.glob(root + '*')
         comps = map(lambda x: os.path.isdir(x) and x + os.sep or x, comps)
         comps = map(lambda x: x[len(expansions[0]):], comps)
@@ -746,20 +759,6 @@ class HashDirective(LeafToken):
     leading # character."""
 
     commencer_ch = "#"
-    # Grouped into mutually-exclusive sets.
-    directives_g = [["exclusive", "piped", "pty"],
-                    ["changeview", "dontchangeview"],
-                    ]
-    directives = reduce(lambda x,y: x+y, directives_g)
-
-    @staticmethod
-    def canonify(dlist):
-        """Given a list of directive names, remove incompatible ones and
-        duplicates, preferring the last of each group."""
-        for dg in HashDirective.directives_g:
-            dlist =  (filter(lambda x: x not in dg, dlist) +
-                      filter(lambda x: x in dg, dlist)[-1:])
-        return dlist
 
     def expand(self):
         """Hash directives are not included in the final command line."""
@@ -772,7 +771,7 @@ class HashDirective(LeafToken):
         if cloc != self.__len__(): # Cursor must be at the end
             return
         root = self.__str__()[1:]
-        comps = filter(lambda x: x.startswith(root),HashDirective.directives)
+        comps = filter(lambda x: x.startswith(root), jobs.directives)
         comps = map(lambda x: x[len(root):] + " ", comps)
         comps.sort()
         return [root] + comps
