@@ -25,7 +25,7 @@ import curses
 
 # Configuration and data directory access for hsh.
 
-version = '1.-2.3'
+version = '1.-2.4'
 
 conf_dir = os.path.join(os.environ["HOME"], ".hsh")
 jobs_dir = os.path.join(conf_dir, "jobs")
@@ -48,25 +48,30 @@ tab_width = 8
 # BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE
 
 faces = {
-    "default":      (curses.COLOR_WHITE,   curses.COLOR_BLACK),
-    "alert":        (curses.COLOR_WHITE,   curses.COLOR_RED),
-    "wrapmark":     (curses.COLOR_CYAN,       curses.COLOR_BLACK),
+    "default":          (curses.COLOR_WHITE,   curses.COLOR_BLACK),
+    "text":             (curses.COLOR_WHITE,   curses.COLOR_BLACK),
+    "wrapmark":         (curses.COLOR_CYAN,    curses.COLOR_BLACK),
+    "stdout":           (curses.COLOR_WHITE,   curses.COLOR_BLACK),
+    "stderr":           (curses.COLOR_RED,     curses.COLOR_BLACK),
+    "stdin":            (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
+    "editbuff":         (curses.COLOR_WHITE,   curses.COLOR_MAGENTA),
+    "header":           (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
+    "headerwrapmark":   (curses.COLOR_CYAN,    curses.COLOR_BLACK),
+    "prefix":           (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
+    "listjobheader":    (curses.COLOR_BLUE,    curses.COLOR_BLACK),
+    "highlight":        (curses.COLOR_WHITE,   curses.COLOR_BLUE),
+    "search":           (curses.COLOR_WHITE,   curses.COLOR_MAGENTA),
 
-    "input.header": (curses.COLOR_YELLOW, curses.COLOR_BLACK),
-    "input.body":   (curses.COLOR_WHITE, curses.COLOR_BLACK),
-    "job.header":   (curses.COLOR_WHITE, curses.COLOR_BLUE),
-    "job.headerwrap":  (curses.COLOR_CYAN,    curses.COLOR_BLUE),
-    "job.stdout":   (curses.COLOR_WHITE,   curses.COLOR_BLACK),
-    "job.stderr":   (curses.COLOR_RED,     curses.COLOR_BLACK),
-    "job.stdin":    (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
-    "job.in_buff":     (curses.COLOR_WHITE, curses.COLOR_MAGENTA),
+    "JobView.header":           (curses.COLOR_WHITE,   curses.COLOR_BLUE),
+    "JobView.headerwrapmark":   (curses.COLOR_CYAN,    curses.COLOR_BLUE),
 
-    "list.header":     (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
-    "list.jobheader":  (curses.COLOR_BLUE,    curses.COLOR_BLACK),
-    "list.highlight":  (curses.COLOR_WHITE,   curses.COLOR_BLUE),
-    "list.highlight2": (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
+    "Alert.text":               (curses.COLOR_WHITE,   curses.COLOR_RED),
 
-    "kv.mod_buff":     (curses.COLOR_WHITE, curses.COLOR_MAGENTA),
+    "SearchView.text":          (curses.COLOR_MAGENTA, curses.COLOR_BLACK),
+    "SearchView.prefix":        (curses.COLOR_WHITE,   curses.COLOR_MAGENTA),
+
+    "CommandHistory.highlight":     (curses.COLOR_YELLOW,  curses.COLOR_BLACK),
+    "CommandHistory.listjobheader": (curses.COLOR_WHITE,   curses.COLOR_BLACK),
 }
 
 # Output formats
@@ -92,27 +97,25 @@ listviews = [
       'header' : "",
       'show_job_output' : True,
       'sync_main_job' : True,
-      'display_format' : "(%(jobid)s) %(username)s@%(hostname)s:%(cwd)s\n"
+      'job_header_fmt' : "(%(jobid)s) %(username)s@%(hostname)s:%(cwd)s\n"
                   "%(state)s> %(expanded_cmd)s" },
 
     { 'name' : "RunningJobs",
       'job_filter' : lambda x: x.get_state() in ["Running", "Initializing"]
                                and x.is_new,
       'header' : "    Running Jobs   \n",
-      'display_format' : "%(username)s@%(hostname)s:%(cwd)s> %(expanded_cmd)s\n"
+      'job_header_fmt' : "%(username)s@%(hostname)s:%(cwd)s> %(expanded_cmd)s\n"
                    "(%(jobid)s) %(state)s - %(total_lines)s lines of output\n"},
 
     { 'name' : "CommandHistory",
       'job_filter' : lambda x: True,
-      'highlight_face' : "list.highlight2",
-      'jobheader_face' : "default",
-      'display_format' : "%(jobid)s %(cmd_line)s" },
+      'job_header_fmt' : "%(jobid)s %(cmd_line)s" },
 
     { 'name' : "JobHistory",
       'job_filter' : 
             lambda x: ((x.get_state() != "NotRun")),
       'header' : "    Full Job History   \n",
-      'display_format' : "%(username)s@%(hostname)s:%(cwd)s> %(expanded_cmd)s\n"
+      'job_header_fmt' : "%(username)s@%(hostname)s:%(cwd)s> %(expanded_cmd)s\n"
                   "(%(jobid)s) %(state)s - %(total_lines)s lines of output\n" },
 
     ]
@@ -133,4 +136,7 @@ default_aliases = {
     'quit' : 'exit',
     'logout' : 'exit',
     'bye' : 'exit',
+
+    'history' : 'hsh_view CommandHistory',
+    'jobs'    : 'hsh_view RunningJobs',
 }
