@@ -56,6 +56,7 @@ from job_view import JobView
 from list_view import ListView
 from message_view import MessageView
 from keyvalue_view import KeyValueView
+from search_view import SearchView
 
 def init_curses():
     stdscr = curses.initscr()
@@ -118,21 +119,18 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
 
     # Private variables
     #    scr: a curses window object for the whole screen
-    #    maxyx: a 2 tuple for the size of the whole screen last time the
-    #           windows were defined.
     #    full_screen_job: If a job has been granted control of the terminal,
     #                     this will be its job object.  If not, it is None.
-
-    # View objects are stored in the self.views dict.
-
+    #    views: a dict storing all view objects indexed by their name.
+    #    layout: a list of view objects which are visible, starting from the
+    #            top of the screen.  layout[0] is the main view.
     #    focus: a View object which currently has the cursor
-    #    main: A View object which is currently in the main view
 
     def __init__(self, stdscr, tsettings):
         self.scr = stdscr
         self.tsettings = tsettings
-        self.maxyx = None
         self.full_screen_job = None
+        self.input_reassigned = False # True means a job is reading input
 
         self.initialize_faces()
 
@@ -143,9 +141,10 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         self.views = {}
         self.views["JobView"] = None
         for v in [InputView(self),
+                  SearchView(self),
                   MessageView(self, "Welcome", hsh.welcome.message),
                   MessageView(self, "TooSmall","Help! The screen is too small"),
-                  MessageView(self, "Alert", "", face="alert"),
+                  MessageView(self, "Alert", "", can_focus=False),
                   MessageView(self, "CrashLog", "Nothing so far!"),
                   KeyValueView(self, "Aliases", hsh.aliases.manager),
                   KeyValueView(self, "Environment", os.environ)
@@ -157,14 +156,15 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
             newview = ListView(self, **newviewconf)
             self.views[newview.get_name()] = newview
 
-        self.input_reassigned = False # True means a job is reading input
+        self.layout = [self.views["Welcome"],
+                       self.views["InputView"],
+                       self.views["Alert"]]
+
         self.focus = None
         self.give_focus(self.views["InputView"])
-        self.main = self.views["Welcome"]
 
         self.displaythread = CursesDisplayThread(self)
         self.displaythread.setDaemon(True)
-        self.displaythread.start()
 
         hsh.jobs.manager.register_listener(self)
 
@@ -179,7 +179,7 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
             logging.debug("COLORS: %d  COLOR_PAIRS: %d" %
                           (curses.COLORS, curses.COLOR_PAIRS))
             logging.debug("can_change_color = %s" % curses.can_change_color())
-        self.faces = {}
+        self._faces = {}
         # Assign a colour pair number to each specified text face.  This
         # restricts us to COLOR_PAIRS different faces.  If this is insufficient
         # a numbering scheme for mapping pairs to numbers could be created, but
@@ -187,25 +187,43 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         cnum = 1
         for faceinfo in hsh.config.faces.items():
             curses.init_pair(cnum, faceinfo[1][0], faceinfo[1][1])
-            self.faces[faceinfo[0]] = curses.color_pair(cnum)
+            self._faces[faceinfo[0]] = curses.color_pair(cnum)
             logging.debug("name=%s cnum=%s attr=%s fi[0]=%s fi[1]=%s" %
-                          (faceinfo[0], cnum, self.faces[faceinfo[0]],
+                          (faceinfo[0], cnum, self._faces[faceinfo[0]],
                            faceinfo[1][0], faceinfo[1][1]))
             cnum += 1
-        logging.debug(str(self.faces))
+
+    def get_face(self, viewname, facename):
+        vn = viewname + "." + facename
+        if vn in self._faces:
+            return self._faces[vn]
+        if facename in self._faces:
+            return self._faces[facename]
+        logging.debug("No face named " + facename)
+        return self._faces["default"]
+
+    def get_size(self):
+        """Return a pair with the size of the display, (height, width)."""
+        return self.scr.getmaxyx()
 
     def get_job_view(self, job):
         if job.jid not in self.job_view_index:
             self.job_view_index[job.jid] = JobView(self, job)
         return self.job_view_index[job.jid]
 
-    def give_focus(self, displayable):
-        if self.focus == displayable:
+    def get_main_view(self):
+        return self.layout[0]
+
+    def give_focus(self, newfocus):
+        if newfocus not in self.layout:
+            raise Exception("Cannot give focus to invisible view")
+        if self.focus == newfocus:
             return
         if self.focus is not None:
             self.focus.set_focus(has_focus=False)
-        self.focus = displayable
-        self.focus.set_focus(has_focus=True)
+        self.focus = newfocus
+        if not self.focus.set_focus(has_focus=True):
+            raise Exception("View refused focus: " + self.focus.get_name())
 
     def handle_crash(self, exc):
         # Clear the input area, in case that was the cause of the issue
@@ -236,6 +254,8 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         # correctly, and using select() for timeouts reading stdin didn't work
         # correctly when curses keystrokes starting with ESC were received.
 
+        self.displaythread.start()
+
         while True:
             try:
                 while self.input_reassigned:
@@ -307,31 +327,45 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         self.escape = False
 
     def next_window(self):
-        # Move focus to the next window
-        if self.focus == self.views["InputView"]:
-            self.give_focus(self.main)
-        else:
-            self.give_focus(self.views["InputView"])
+        """Move focus to the next view."""
+        # Rotate layout into a list of candidates for accepting focus.
+        cands = list(self.layout)
+        if self.focus in cands:
+            while cands[-1] != self.focus:
+                cands.append(cands.pop(0))
 
-    def set_main(self, cd, refocus=False):
-        # Set the provided displayable object as the main window.
-        if cd is None:
-            cd = self.views["Welcome"]
-        logging.debug("Switching display to %s" % cd.get_name())
+        # Try to change focus
+        self.focus.set_focus(has_focus=False)
+        for view in cands:
+            if view.set_focus(has_focus=True):
+                self.focus = view
+                return
+
+    def set_main(self, newmain, give_focus=False):
+        # Show the provided view object in the main area.
+        if newmain is None:
+            newmain = self.views["Welcome"]
+        logging.debug("Switching display to %s" % newmain.get_name())
         self.show_alert("")
-        if self.focus == self.main or refocus:
-            self.give_focus(cd)
-        if cd != self.main:
-            self.main = cd
+        if self.focus == self.layout[0]:
+            give_focus = True
+        self.layout[0] = newmain
+        if give_focus:
+            self.give_focus(newmain)
 
     def display_job(self, job, show=None, focus=False):
-        """Set the job being displayed, if show is False, then the job
-        displayable is updated, but the job is not shown.  job may be a
-        hsh.jobs.Job object, a JobView object, -1 to mean move back a
-        job, +1 to mean move forward a job, or 0 to mean the current one is
-        gone, and a new one needs to be chosen."""
+        """Set the job in the current Job View.
+
+        @param job: hsh.jobs.Job or JobView object; -1 to move back a job; +1 to
+        move forward a job; or 0 to mean the current one is gone, and
+        a new one needs to be chosen.
+
+        @param show: True means show the new job; False means don't change the
+        view; None (default) means show the new job if JobView is visible.
+
+        @param focus: boolean indicating if jobview should receive focus."""
         if show == None:
-            show = self.views["JobView"] == self.main
+            show = self.views["JobView"] == self.layout[0]
         if type(job) == int:
             jv = self.views["JobView"]
             sl = self.views["SessionList"]
@@ -356,16 +390,37 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         # job is now a JobView or None
         self.views["JobView"] = job
         if show:
-            self.set_main(job, refocus=focus)
+            self.set_main(job, give_focus=focus)
             jobid = job and job.job or None
             for view in self.views.values():
                 if view is not None:
                     view.set_visible_job(jobid)
+                    return
 
     def display_sessionlist(self):
         "Show the session list when input has created a new job."
         self.set_main(self.views["SessionList"])
 
+    def display_search(self, show=None):
+        """Display the search window per user request, boolean argument
+        indicates if search should be displayed or hidden, default or None
+        means toggle."""
+        sv = self.views["SearchView"]
+        if show == None:
+            show = not sv in self.layout
+        if show == False and sv in self.layout:
+            self.layout.remove(sv)
+            if self.focus == sv:
+                if self.presearch_focus in self.layout:
+                    self.focus = self.presearch_focus
+                else:
+                    self.focus = self.layout[0]
+        if show == True:
+            if sv not in self.layout:
+                self.layout.insert(1, sv)
+                self.presearch_focus = self.focus
+            self.focus = sv
+
     def relinquish_terminal(self, job):
         # Deinitialize curses so that a subprocess can take control of the
         # terminal.  If the process is being brokered through a tty or pipe,
@@ -403,8 +458,8 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
             self.input_reassigned = False
         self.full_screen_job = None
         self.scr.clear()
-        self.main.set_dirty()
-        self.views["InputView"].set_dirty()
+        for view in self.layout:
+            view.set_dirty()
         curses.flushinp()
 
     ######################################################################
@@ -424,12 +479,12 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
     def get_visible_job(self):
         """Return whichever job the user would think is the current one, based
         on the visible display."""
-        return self.main.current_job()
+        return self.layout[0].current_job()
 
     def show_view(self, view_name):
         """Show the view of the given name to the user."""
         if view_name in self.views:
-            self.set_main(self.views[view_name], refocus=True)
+            self.set_main(self.views[view_name], give_focus=True)
         else:
             self.show_alert("No view with that name")
 
@@ -452,100 +507,120 @@ class CursesDisplayThread(threading.Thread):
         super(CursesDisplayThread, self).__init__()
         self.display = display
 
-    def _ensure_size(self):
-        """Ensure that the screen is large enough to be useful.  If it's too
-        small, show a message and wait for it to be bigger.  Return the screen
-        height and width, or (0,0) if it had to wait for a larger size."""
-        d = self.display
-        s = d.scr
-        min_w = max(d.focus.min_width(), d.views["InputView"].min_width())
-        min_h = (d.focus.min_height() + d.views["InputView"].min_height()
-                 + d.views["Alert"].min_height())
-        (h, w) = s.getmaxyx()
-        old_focus = d.focus
+    def _ensure_size(self, layout, h, w):
+        """Check that the screen is large enough to be useful.  If it's too
+        small, show a message and wait for it to be bigger.  Return true if it
+        was big enough first time, and false otherwise."""
+        min_w = max(map(lambda x: x.min_width(), layout))
+        min_h = reduce(lambda x,y: x+y, map(lambda x: x.min_height(), layout))
 
-        while (s.getmaxyx()[0] < min_h or s.getmaxyx()[1] < min_w):
-            (h, w) = (0, 0)
-            d.give_focus(d.views["TooSmall"])
-            d.views["TooSmall"].draw(s.subwin(0, 0))
+        (h, w) = self.display.scr.getmaxyx()
+        if (h >= min_h) and (w >= min_w):
+            return True
+
+        # Screen is too small, so wait for it to be bigger
+        old_focus = self.display.focus
+        old_layout = self.display.layout
+
+        while (h < min_h) or (w < min_w):
+            self.display.layout = [self.display.views["TooSmall"]]
+            self.display.give_focus(self.display.views["TooSmall"])
+            self.display.views["TooSmall"].draw(self.display.scr.subwin(0, 0))
             time.sleep(0.1)
+            (h, w) = self.display.scr.getmaxyx()
 
-        d.give_focus(old_focus)
+        self.display.layout = old_layout
+        self.display.give_focus(old_focus)
 
-        return (h, w)
+        return False
 
-    def _partition_screen(self, h, w):
+    def _partition_screen(self, layout, h, w):
         """Break the screen up into several windows of the necessary sizes."""
-        d = self.display
-        s = d.scr
+        scr = self.display.scr
+        curses_wins = []
+        for view in reversed(layout[1:]):
+            vh = view.min_height()
+            h -= vh
+            curses_wins.insert(0, scr.subwin(vh, w, h, 0))
+        curses_wins.insert(0, scr.subwin(h, w, 0, 0))
 
-        # The alert window appears at the very bottom of the screen, the
-        # input window above it, and the rest is for the main display.
-        ah = d.views["Alert"].min_height()
-        h = h - ah
-        self.alert_win = s.subwin(ah, w, h, 0)
-
-        inh = d.views["InputView"].min_height()
-        h = h - inh
-        self.input_win = s.subwin(inh, w, h, 0)
-
-        self.main_win = s.subwin(h, w, 0, 0)
+        return curses_wins
 
     def run(self):
         disp = self.display
 
-        last_main_view = None  # The last main view which was drawn
-        (h, w) = (None, None)  # The curses size at last draw
+        # Remember the view layout and curses size from the last draw cycle.
+        prev_layout = []
+        (prev_h, prev_w) = (None, None)
+        curses_wins = []  # curses windows corresponding to layout.
 
         while True:
             stdscr = disp.scr
             try:
-                force = False
-                main_view_changed = False
-                drawn = False
+                force_redraw = False  # If changes force a redraw this time.
+                drawn = False         # If something was drawn this time.
 
-                if (h, w) != self._ensure_size():
-                    # The screen has changed size, so redraw everything.
-                    (h, w) = stdscr.getmaxyx()
-                    force = True
-                    self._partition_screen(h, w)
+                # Check for layout and size changes.
+                while True:
+                    # Make copies of layout and size so nothing changes mid-draw
+                    layout = list(disp.layout)
+                    (h, w) = disp.scr.getmaxyx()
+                    focus = disp.focus
 
-                main_view = disp.main
-                input_view = disp.views["InputView"]
-                alert_view = disp.views["Alert"]
-                focus_view = disp.focus
+                    if not self._ensure_size(layout, h, w):
+                        # Had to wait for a size change, so previous size is
+                        # now wrong.
+                        (prev_h, prev_w) = (None, None)
+                        # Continue loop to pick up layout and size changes.
+                        continue
 
-                if last_main_view != main_view:
-                    main_view_changed = True
-                    last_main_view = main_view
+                    break
 
-                if focus_view.wants_terminal() != 0:
+                # Handle full screen jobs
+                if focus.wants_terminal() != 0:
                     # The focus view has requested the full screen
-                    disp.relinquish_terminal(focus_view)
-                    focus_view.set_has_terminal(True)
-                    while (disp.focus == focus_view and
-                           focus_view.wants_terminal()):
+                    disp.relinquish_terminal(focus)
+                    focus.set_has_terminal(True)
+                    while (disp.focus == focus and
+                           focus.wants_terminal()):
                         time.sleep(0.1)
-                    focus_view.set_has_terminal(False)
+                    focus.set_has_terminal(False)
                     disp.claim_terminal()
-                    (h, w) = (None, None) # force a redraw
+                    (prev_h, prev_w) = (None, None) # force a redraw
                     continue
 
-                d1 = main_view.draw(self.main_win, force or main_view_changed)
-                d2 = input_view.draw(self.input_win, force)
-                d3 = alert_view.draw(self.alert_win, force)
-                drawn = d1 or d2 or d3
+                # Recalculate curses windows if necessary
+                if (prev_h,prev_w) != (h,w) or prev_layout != layout:
+                    force_redraw = True
+                    curses_wins = self._partition_screen(layout, h, w)
+
+                prev_layout = layout
+                (prev_h, prev_w) = (h, w)
+
+                # If search is active and has changed since last time then
+                # force a redraw.
+                sv = None
+                if self.display.views["SearchView"] in layout:
+                    sv = self.display.views["SearchView"]
+                    if sv.is_dirty():
+                        force_redraw = True
+
+                for i in range(len(layout)):
+                    d1 = layout[i].draw(curses_wins[i], force_redraw, search=sv)
+                    sv = None  # Only highlight searches in first view
+                    if d1:
+                        drawn = True
 
                 if drawn:
-                    # It's difficult to prevent the cursor from moving while
-                    # drawing, so draw it last.
-                    if focus_view == main_view:
-                        main_view.draw_cursor(self.main_win)
-                    if focus_view == input_view:
-                        input_view.draw_cursor(self.input_win)
+                    # Draw the cursor in case a broken drawing routine moved it.
+                    for i in range(len(layout)):
+                        if layout[i] == focus:
+                            layout[i].draw_cursor(curses_wins[i])
                 else:
                     # Take a pause if nothing needed to be drawn.
                     time.sleep(0.1)
 
             except Exception, e:
                 self.display.handle_crash(e)
+                # Don't spin when dealing with an exception
+                time.sleep(1)
