@@ -21,8 +21,10 @@
 
 # The format of the user configuration file is to be determined.
 
+import curses
 import logging
 import os
+import re
 import sys
 import time
 import traceback
@@ -32,31 +34,24 @@ import hsh.config
 import hsh.jobs
 
 from hsh.exceptions import *
+from hsh.lib import *
 from view import View
 from content_view import ContentView
 
 class JobView(ContentView, hsh.jobs.JobListener):
 
     def __init__(self, display, job):
-        super(JobView, self).__init__(display)
+        super(JobView, self).__init__(display, job.get_output())
         self.job = job
-        self.content = job.get_output()
 
         self.header_fmt = hsh.config.header_job
-        self.header_face = self.display.faces["job.header"]
-        self.header_wrapface = self.display.faces["job.headerwrap"]
-        self.header_wrap = True
+        self.header_formatter.set_format(wrap = True)
         # header_size will change, but this is a good start.
         self.header_size = len(self.header_fmt.split('\n'))
 
         self.job.register_listener(self)
 
         self.raw_output = False
-        self.output_tail = False
-
-        # show_output controls how the job output is shown in some list views.
-        # -1 means show all, larger integers mean show that many lines.
-        self.show_output = -1
 
         if self.job.can_send_input() or self.job.state == "Initializing":
             self.input_mode = "silent"
@@ -75,6 +70,22 @@ class JobView(ContentView, hsh.jobs.JobListener):
             self.move_bottom(None)
         return super(JobView, self).draw(*args, **kwargs)
 
+    def draw_cursor(self, win):
+        if self.input_mode not in ["character", "line"]:
+            return super(JobView, self).draw_cursor(win)
+
+        # During input modes, the cursor needs to go right at the end, and
+        # ideally would be able to move around in the edit buffer.
+        cy, cx = self.textf.get_end_pos()
+        cy = clamp(0, cy, self.cur_height - 1)  # workaround position bugs 
+        # I want to use win.move(cy, cx), but it doesn't work for me.
+        (py, px) = win.getparyx()
+        # Account for the header size, and place cursor after text
+        self.display.scr.move(py + cy + self.header_size, px + cx + 1)
+
+        curses.curs_set(2)  # Make the cursor visible again.
+        self.display.scr.refresh()
+
     ######################################################################
     # View interface
 
@@ -84,12 +95,12 @@ class JobView(ContentView, hsh.jobs.JobListener):
               'cwd'      : self.job.cwd,
               'cmd_line' : self.job.cmdline,
               'expanded_cmd' : hsh.command.format_cmd(self.job.cmd),
-              'display_line' : self.display_pos[0] + 1,
-              'total_lines'  : len(self.content),
+              'display_line' : self.textf.display_pos[0] + 1,
+              'total_lines'  : len(self.text),
               'jobid'        : self.job.jid,
               'pid'          : self.job.pid,
               'input_mode'   : self.input_mode,
-              'output_tail'  : self.output_tail and "tailing" or "",
+              'output_tail'  : self.textf.get_tail() and "tailing" or "",
               }
         hi['state'] = self.job.get_state()
         if self.job.get_retcode():
@@ -111,11 +122,11 @@ class JobView(ContentView, hsh.jobs.JobListener):
 
     def set_focus(self, has_focus):
         if self.has_focus == has_focus:
-            return
+            return True
         self.set_dirty()
         self.has_focus = has_focus
         if self.wants_terminal() != 2:
-            return
+            return True
         # Exclusive jobs require more handling when switching focus
         if self.has_focus:
             # Exclusive job is receiving focus, so start it running.
@@ -128,12 +139,7 @@ class JobView(ContentView, hsh.jobs.JobListener):
             # call can't come from the main thread, because exclusive jobs
             # block it.  Presumably it's an error in another thread.
             self.job.suspend()
-
-    def set_face(self, region):
-        """Given a region, set its face."""
-        super(JobView, self).set_face(region)
-        if "source" in region.__dict__:
-            region.face = self.display.faces["job." + region.source]
+        return True
 
     def current_job(self):
         return self.job
@@ -160,8 +166,6 @@ class JobView(ContentView, hsh.jobs.JobListener):
 
     def on_job_output(self, job):
         self.set_dirty("append")
-        if self.output_tail:
-            self.move_bottom(None)
 
     def on_job_terminate(self, job):
         self.input_mode = "closed"
@@ -177,11 +181,7 @@ class JobView(ContentView, hsh.jobs.JobListener):
 
     def restart(self):
         """Restart this job."""
-        self.cursor_pos = [0,0]
-        self.display_pos = [0,0]
-        if "exclusive" in self.job.get_directives():
-            # For exclusive jobs, start() blocks until the job is visible.
-            self.display.display_job(self.job, focus=True)
+        self.textf.move_top()
         self.job.start()
         if self.job.get_state() == "Suspended":
             self.display.next_window()
@@ -199,18 +199,13 @@ class JobView(ContentView, hsh.jobs.JobListener):
     #   fullscreen:   process uses curses and requires the full terminal screen
     #   closed:       the input pipe has been closed
 
-    # Output modes
-
-    # self.output_tail is a boolean, if True then the display will be updated to
-    #                  always show the bottom line of output.
-
     def _align_inbuff(self):
         # Move the line input buffer to the end of the display
         if self.input_buffer is not None:
             buf_txt = str(self.input_buffer)
             self.input_buffer.delete()
-            self.input_buffer = self.content.append_region(buf_txt)
-            self.input_buffer.source = "in_buff"
+            self.input_buffer = self.text.append_region(buf_txt)
+            self.input_buffer.source = "editbuff"
 
     # Commands bound to keystrokes
 
@@ -218,11 +213,13 @@ class JobView(ContentView, hsh.jobs.JobListener):
         """Cycle through the various input options."""
         if self.input_mode == "silent":
             self.input_mode = "line"
+            self.textf.set_tail(True)
         elif self.input_mode == "line":
             if self.input_buffer is not None:
                 self.input_buffer.delete()
                 self.input_buffer = None
             self.input_mode = "character"
+            self.textf.set_tail(True)
         elif self.input_mode == "character":
             self.input_mode = "silent"
         elif self.input_mode == "closed" and self.job.can_send_input():
@@ -238,17 +235,16 @@ class JobView(ContentView, hsh.jobs.JobListener):
         self.input_mode = "closed"
 
     def toggle_tail(self, ki):
-        self.output_tail = not self.output_tail
+        self.textf.set_tail(not self.textf.get_tail())
 
     def insert(self, ki):
         if self.input_mode == "character":
-            inch = self.content.append_region(ki.ch())
-            inch.source = "stdin"
+            self.text.append_region(ki.ch(), {"source" : "stdin"})
             self.job.send_input(ki.ch())
         elif self.input_mode == "line":
             if self.input_buffer is None:
-                self.input_buffer = self.content.append_region(ki.ch())
-                self.input_buffer.source = "in_buff"
+                self.input_buffer = self.text.append_region(ki.ch())
+                self.input_buffer.source = "editbuff"
             else:
                 self._align_inbuff()
                 self.input_buffer.extend(ki.ch())
@@ -260,7 +256,7 @@ class JobView(ContentView, hsh.jobs.JobListener):
             self.insert(ki)
         elif self.input_mode == "line":
             if self.input_buffer is None:
-                self.input_buffer = self.content.append_region(ki.ch())
+                self.input_buffer = self.text.append_region(ki.ch())
             else:
                 self._align_inbuff()
                 self.input_buffer.extend(ki.ch())
@@ -293,3 +289,22 @@ class JobView(ContentView, hsh.jobs.JobListener):
     def restart_job(self, ki):
         self.restart()
 
+    def edit_job(self, ki):
+        iv = self.display.views["InputView"]
+        iv.cancel(ki)
+        orig_cmdline = self.job.cmdline
+        repl_str = "#replace" + str(self.job.jid)
+        # Find the first unescaped space
+        first_space_re = re.compile(r"^(\\.|[^\\ ])* ")
+        fs_match = first_space_re.search(orig_cmdline)
+        if fs_match is None:
+            cmdline = orig_cmdline + " " + repl_str + " "
+        else:
+            fsl = fs_match.end()
+            cmdline = orig_cmdline[0:fsl] + repl_str + " "
+            # Remove duplicate replace clauses
+            if orig_cmdline[fsl:].startswith(repl_str + " "):
+                fsl += len(repl_str + " ")
+            cmdline += orig_cmdline[fsl:]
+        iv.insert_text(cmdline)
+        self.display.give_focus(iv)
