@@ -22,7 +22,7 @@
 import curses
 import logging
 
-import hsh.content
+import hsh.content.text
 
 from view import View
 
@@ -61,7 +61,6 @@ class KeyValueView(View):
         self.cur_width = 1
 
         self.header_fmt = "    %s\n" % name
-        self.header_face = self.display.faces["list.header"]
 
         # To handle editing, the modified text is stored separately, and only
         # written to the dict when editing is finished.
@@ -134,7 +133,7 @@ class KeyValueView(View):
         curses.curs_set(2)  # Make the cursor visible again.
         self.display.scr.refresh()
 
-    def draw(self, win, force_redraw):
+    def draw(self, win, force_redraw, search=None):
         
         if (not force_redraw and not self.is_dirty() and 
             self.curkey == self.last_curkey):
@@ -150,24 +149,30 @@ class KeyValueView(View):
         cwin.clear()
 
         (self.cur_height, self.cur_width) = (h, w) = cwin.getmaxyx()
-        fface = None
+        search_face = self.get_face("search")
+        face_attrs = None
         if self.edit_buf is not None:
-            fface = self.display.faces["kv.mod_buff"]
+            face_attrs = {"face" : self.get_face("editbuff")}
 
         def append_txt(tl, txt, width, dp, face=None):
             """Append the string txt to the TextLine tl, using at most width
             characters, starting from character dp."""
-            rg = tl.append_region(dp > 0 and "_" or " ")
-            rg.face = self.display.faces["wrapmark"]
-            txta = txt[dp:dp+width - 2]
-            if len(txta) > 0:
-                rg = tl.append_region(txta)
-                if face:
-                    rg.face = face
-            rg = tl.append_region((len(txt) - dp >= width - 1) and "_" or " ")
-            rg.face = self.display.faces["wrapmark"]
-            if (width-2) - len(txta) > 0:
-                tl.append_region(" "*((width-2) - len(txta)))
+            tl.append_region(dp > 0 and "_" or " ",
+                             attrs = {"face" : self.get_face("wrapmark")})
+            txtl = hsh.content.text.TextLine()
+            txtl.append_region(txt, attrs=face)
+            if search:
+                txtl = search.highlight_regions(txtl, search_face)
+
+            if dp < len(txtl):
+                txtl[0:dp] = []
+                txtl[width - 2:] = []
+                for region in txtl.regions:
+                    tl.append_region(region, attrs=face)
+            tl.append_region((len(txt) - dp >= width - 1) and "_" or " ",
+                             attrs = {"face" : self.get_face("wrapmark")})
+            if (width-2) - len(txtl) > 0:
+                tl.append_region(" "*((width-2) - len(txtl)))
 
         # Draw the keys and values
         dmin = self.dispkey or 0
@@ -177,11 +182,11 @@ class KeyValueView(View):
         for i in range(dmin,dmax):
             k = ks[i]
             v = self.content_dict[k]
-            tl = hsh.content.TextLine()
+            tl = hsh.content.text.TextLine()
             if self.curkey == i and self.cursor_pos[0] == 0:
                 t = self.edit_buf
                 if t is None: t = k
-                append_txt(tl, t, self.kw, self.cursor_pos[2], face=fface)
+                append_txt(tl, t, self.kw, self.cursor_pos[2], face=face_attrs)
             else:
                 append_txt(tl, k, self.kw, 0)
             tl.append_region(" ")
@@ -189,7 +194,7 @@ class KeyValueView(View):
                 t = self.edit_buf
                 if t is None: t = v
                 append_txt(tl, t, self.cur_width - self.kw - 1,
-                           self.cursor_pos[2], face=fface)
+                           self.cursor_pos[2], face=face_attrs)
             else:
                 append_txt(tl, v, self.cur_width - self.kw - 1, 0)
 
