@@ -21,7 +21,7 @@
 
 # The format of the user configuration file is to be determined.
 
-import hsh.content
+import hsh.content.text
 
 from hsh.exceptions import *
 from view import View
@@ -29,17 +29,16 @@ from content_view import ContentView
 
 class MessageView(ContentView):
 
-    def __init__(self, display, name, message, face="default"):
-        super(MessageView, self).__init__(display, name)
-        self.face = face
-        self._set_message(message)
-
-    def _set_message(self, message):
-        self.content = hsh.content.TextContent()
-        for line in message.split('\n'):
-            r = self.content.append_region(line + '\n')
-            r.face = self.display.faces[self.face]
+    def __init__(self, display, name, message, can_focus=True):
+        text = hsh.content.text.TextList(message.split('\n'))
+        super(MessageView, self).__init__(display, text, name)
+        self.can_focus = can_focus
 
     def set_message(self, message):
-        self._set_message(message)
+        self.text[:] = message.split('\n')
         self.set_dirty()
+
+    def set_focus(self, *args, **kwargs):
+        if not self.can_focus:
+            return False
+        return super(MessageView, self).set_focus(*args, **kwargs)
