@@ -24,7 +24,7 @@ import logging
 
 import hsh
 import hsh.config
-import hsh.content
+import hsh.content.text
 
 from hsh.exceptions import *
 
@@ -67,171 +67,6 @@ class View(object):
     and the header_info() method, or just overriding the draw_header() method.
     """
 
-    @staticmethod
-    def format_lines(display, lines, height, width, dpos=[0,0], wrap=True,
-                     face=None, wrapface=None, trim=0, tail=False):
-        """Format the provided lines into lines of appropriate size for display.
-          lines is a TextContent or a list of NL terminated strings
-          wrap  is a boolean, if True, lines should be wrapped
-          dpos  is a pair of ints indicating where the display should start
-                in the provided lines.  The first is the vertical position,
-                from the top line down, the second is the horizontal position
-                from left to right.
-          width is the width to make the lines
-          height is the number of output lines to generate.  If the last line
-                needs to be wrapped, more than height lines will be returned
-                so that the final input line will be completely included.
-          face is the face to use for output lines.  If None and lines is a
-               TextContent, then the face from lines will be used, otherwise
-               no face will be set.
-          wrapface is the face used for line wrap indicators.
-          trim is the number of characters to trim from the left of each line.
-
-          tail is a boolean defaulting to False, if True formatting will start
-               at the end of lines, working up, and dpos's will count lines
-               from the bottom, and characters from the right.
-
-        A TextContent object is returned, preserving any formatting regions of
-        the original source.  Additionally, each TextLine within the content
-        has its "line_number" variable set to the number of the line from which
-        it originated.
-
-        A blank column is included at the beginning and end of each formatted
-        line, and line continuation or truncation markers are put in that
-        column, using the specified wrapface.
-        """
-
-        content_fmt = hsh.content.TextContent()
-        wrap_width = width - 1
-
-        if not wrapface:
-            wrapface=display.faces["wrapmark"]
-
-        if not tail:
-            # Start formatting from the first line and append
-            lnum = dpos[0]
-            while len(content_fmt) < height and lnum < len(lines):
-                line = lines[lnum]
-                ltrim = (lnum == dpos[0]) and max(trim, dpos[1]) or trim
-
-                line_fmt = View._format_line(line, wrap_width, wrap, face,
-                                             wrapface, ltrim, lnum)
-
-                content_fmt.extend(line_fmt)
-                lnum += 1
-
-        else:
-            # Start formatting from the last line and prepend
-            lnum = len(lines) - dpos[0] - 1
-            while len(content_fmt) < height and lnum >= 0:
-                line = lines[lnum]
-                if dpos[1] > 0 and lnum == (len(lines) - dpos[0]):
-                    line = line[0:len(line) - dpos[1]]
-                line_fmt = View._format_line(line, wrap_width, wrap, face,
-                                             wrapface, trim, lnum)
-
-                content_fmt[:0] = line_fmt
-                lnum -= 1
-
-        return content_fmt
-
-    @staticmethod
-    def _format_line_wrap(content_fmt, wrapface, lc):
-        "Internal function for _format_line()"
-        rg = content_fmt.append_region('\\\n')
-        rg.face = wrapface
-        rg = content_fmt.append_region('/')
-        rg.face = wrapface
-        rg.get_line().line_number = lc
-
-    @staticmethod
-    def _format_line_trunc(content_fmt, wrapface):
-        "Internal function for _format_line()"
-        rg = content_fmt.append_region('_')
-        rg.face = wrapface
-
-    @staticmethod
-    def _format_line_start(content_fmt, wrapface, lc, trim):
-        "Internal function for _format_line()"
-        if trim > 0:
-            rg = content_fmt.append_region('-')
-        else:
-            rg = content_fmt.append_region(' ')
-        rg.face = wrapface
-        rg.get_line().line_number = lc
-
-    @staticmethod
-    def _format_line_end(content_fmt):
-        "Internal function for _format_line()"
-        rg = content_fmt.append_region('\n')
-
-    @staticmethod
-    def _format_line(line, wrap_width, wrap,
-                     face, wrapface, trim, lnum):
-        """Internal function used by format_lines() to format a single line.
-
-        line - A single line to format
-        wrap_width - maximum length of lines, excluding wrap characters.
-        wrap - Boolean if line should wrap or truncate
-        face - may be None, to set text face
-        wrapface - face to use for special wrap characters
-        trim - number of characters to trim from the beginning of the line
-        lnum - Line number to be attached to resulting text content.
-
-        Returns a TextContent object, possibly with multiple lines.
-        """
-
-        content_fmt = hsh.content.TextContent()
-
-        if not isinstance(line, hsh.content.TextLine):
-            line = hsh.content.TextLine(str(line).strip('\n') + '\n')
-
-        # Traverse the line, replacing any tab characters with spaces,
-        # and removing any carriage returns.
-        tw = hsh.config.tab_width
-        i = 0
-        while i < len(line):
-            if line[i] == '\r':
-                line[i:i+1] = ''
-            if line[i] == '\t':
-                sc = (i / tw + 1) * tw - i
-                line[i:i+1] = " " * sc
-                i += sc
-            else:
-                i += 1
-
-        View._format_line_start(content_fmt, wrapface, lnum, trim)
-        for region in line.regions:
-            if len(region) <= trim:
-                trim -= len(region)
-                continue
-
-            rg = content_fmt.append_region(region)
-            if trim > 0:
-                rg[0:trim] = []
-                trim = 0
-
-            if face:
-                rg.face = face
-            if content_fmt[-1][-1] == '\n':
-                content_fmt[-1][-1:] = []
-            while len(content_fmt[-1]) > wrap_width:
-                spill = "".join(content_fmt[-1][wrap_width:])
-                content_fmt[-1][wrap_width:] = []
-                if wrap:
-                    View._format_line_wrap(content_fmt, wrapface, lnum)
-                    rg = content_fmt.append_region(spill)
-                    if face:
-                        rg.face = face
-                    rg.set_attrs(region)
-                else:
-                    View._format_line_trunc(content_fmt, wrapface)
-                    break
-
-        View._format_line_end(content_fmt)
-
-        return content_fmt
-
     def __init__(self, display, name=None):
         self.display = display
 
@@ -242,17 +77,20 @@ class View(object):
         self.has_focus = False
         self.dirty = None
 
+        if name is not None:
+            self.name = name
+
         # The header is a section at the top of display showing information
         # about the view, typically in a different colour, and drawn by the
         # draw_header().
         self.header_fmt = ""
-        self.header_wrap = False
-        self.header_face = None
-        self.header_wrapface = None
         self.header_size = 0 # Set by draw_header()
 
-        if name is not None:
-            self.name = name
+        self.header_formatter = hsh.content.text_formatter.TextFormatter([])
+        self.header_formatter.set_format(wrap = False,
+                                         face_default = "header",
+                                         face_wrap = "headerwrapmark",
+                                         face_view_name = self.get_name())
 
         # Build up the keybindings for this displayable.  Start with the
         # default bindings, and then override with customizations.
@@ -354,14 +192,18 @@ class View(object):
         return self.min_width_main
 
     def set_focus(self, has_focus):
-        # Called when a displayable receives or loses focus
+        """Used by display to inform a View that it is getting or losing focus,
+        as indicated by the boolean has_focus.  Return True if the change is
+        accepted, False otherwise."""
         if self.has_focus != has_focus:
             self.set_dirty()
         self.has_focus = has_focus
+        return True
 
     def set_visible_job(self, job):
-        # Called when the main visible job has changed, in case the view wants
-        # to update internal structures to acknowledge this.
+        """Called by the display object to inform each view that a new job is
+        the main visible job.  The SessionList in particular updates itself
+        based on this information."""
         pass
 
     def header_info(self):
@@ -377,6 +219,10 @@ class View(object):
             return
         self.dirty = newdirty
 
+    def get_face(self, facename):
+        """Return the face for the given name specific to this view."""
+        return self.display.get_face(self.get_name(), facename)
+
     def draw_cursor(self, win):
         """Default implementation does not draw a cursor."""
         try:
@@ -402,10 +248,9 @@ class View(object):
             return
 
         # Format the header into flines
+        self.header_formatter.set_format(width=width)
         rawh = (self.header_fmt % self.header_info()).split('\n')
-        flines = View.format_lines(self.display, rawh, 5, width,
-                                   wrap=self.header_wrap, face=self.header_face,
-                                   wrapface=self.header_wrapface)
+        flines = self.header_formatter.get_lines(5, text=rawh)
 
         # If the header changed size, the rest of the window needs a redraw.
         new_header_size = min(height - 1, 5, len(flines))
@@ -417,25 +262,21 @@ class View(object):
         hwin = win.derwin(self.header_size, width, 0, 0)
         hwin.leaveok(1)
         hwin.clear()
-        hwin.bkgd(' ', self.header_face)
+        hwin.bkgd(' ', self.header_formatter._get_face(formatname="face_default"))
         for i in range(self.header_size):
             self.draw_line(flines[i], i, hwin)
         hwin.refresh()
         hwin.leaveok(0)
 
-    def set_face(self, region):
-        """Given a region, set its face.  This default implementation sets
-        the default face from the display."""
-        region.face = self.display.faces["default"]
-
     # Internal function for drawing each line, already formatted for width.
     def draw_line(self, line, yoff, win):
         try:
             xoff = 0
             for region in line.regions:
-                if "face" not in region.__dict__:
-                    self.set_face(region)
-                face = region.face
+                if "face" in region.__dict__:
+                    face = region.face
+                else:
+                    face = self.display._faces["default"]
                 win.addstr(yoff, xoff, str(region).strip('\n'), face)
                 xoff += len(region)
         except Exception, e:
@@ -455,13 +296,20 @@ class View(object):
         portion."""
         return win.derwin(self.header_size, 0)
 
-    def draw(self, win, force_redraw):
+    def draw(self, win, force_redraw, search=None):
         """This default implementation of draw() is purely illustrative, and
         only draws the header.  Subclasses will certainly override it.
 
-        A boolean is returned, True meaning something was drawn or the cursor
+        @param force_redraw: If true the view object shouldn't try to optimize.
+
+        @param search: a regular expression object whose matches should be
+        highlighted.
+
+        @return: True means something was drawn or the cursor
         position has changed.  This result is used by the main drawing loop to
-        decide whether to draw the cursor, or take a nap."""
+        decide whether to draw the cursor, or take a nap.
+        @rtype: boolean
+        """
         # Unfortunately this code will be duplicated in other draw() methods,
         # because the difficulty of sharing it is too great.
         if force_redraw:
@@ -488,6 +336,16 @@ class View(object):
         is nothing appropriate."""
         return None
 
+    ######################################################################
+
+    def move_search(self, pattern, forward=True, from_end=False):
+        """Given a regular expression object, move the position of the cursor
+        to its next match.  If forward is False, then move to the next search
+        backwards.  If from_end is true, then search from the top or bottom
+        rather than the current cursor position.  Return True if a match was
+        found."""
+        return False
+
     ######################################################################
     # Commands bound to key strokes.
 
@@ -507,10 +365,16 @@ class View(object):
     def restart_job(self, ki):
         pass
 
+    def edit_job(self, ki):
+        pass
+
     def change_view(self, ki, name):
         if name:
             self.display.show_view(name)
 
+    def show_search(self, ki):
+        self.display.display_search(show=True)
+
     def quit(self, ki):
         raise QuitException()
 
@@ -561,3 +425,6 @@ class View(object):
 
     def paste(self, ki):
         pass
+
+    def toggle_wrap(self, ki):
+        pass
