@@ -24,3 +24,6 @@ class QuitException(Exception):
 
 class ConfigException(Exception):
     """Raised when there's an error in user configuration."""
+
+class JobCreationException(Exception):
+    """Raised when an error occurs creating a job."""
