@@ -50,6 +50,7 @@ import fcntl
 import logging
 import os
 import pty
+import re
 import signal
 import subprocess
 import sys
@@ -59,10 +60,15 @@ import thread
 import threading
 import traceback
 
+import hsh
 import config
 import system
-import content
-import command
+import content.text
+
+from hsh.exceptions import *
+
+######################################################################
+# Static dictionaries and lists
 
 # A mapping from command names to builtin classes.
 loaded_builtins = dict()
@@ -78,6 +84,23 @@ def register_builtin(name, biclass):
 
     builtin_classes[biclass.__name__] = biclass
 
+# Job directives, grouped into mutually-exclusive sets.
+directives_g = [["exclusive", "piped", "pty"],
+                ["changeview", "dontchangeview"],
+                ["replace"],
+                ]
+directives = reduce(lambda x,y: x+y, directives_g)
+
+def canonify_directives(dlist):
+    """Given a list of directive names, remove incompatible ones and
+    duplicates, preferring the last of each group."""
+    for dg in directives_g:
+        dlist =  (filter(lambda x: x not in dg, dlist) +
+                  filter(lambda x: x in dg, dlist)[-1:])
+    return dlist
+
+######################################################################
+
 class JobManager(object):
     # Provides access to all known jobs.  Used to create jobs, and assign them
     # ids.  Will read jobs from directories on disk, as well as clean up disk.
@@ -122,28 +145,56 @@ class JobManager(object):
         if listener in self.listeners:
             self.listeners.remove(listener)
 
-    def create_job(self, cmd, cmdline, directives, state):
-        """Create a new job from a command and parameters, assign it an id and
-        return it.
-        cmd: A list of strings: the job to execute followed by its args.
-        cmdline: The string entered by the user, as a reference, but not used
-                 in job creation.
-        state: A string with the job's initial state
-        directives: A list of directives for how to start the job."""
-        if cmd[0] in loaded_builtins:
-            jobcls = loaded_builtins[cmd[0]]
+    def create_job(self, command, start):
+        """Create a new job from a command object and return it.  Start the job
+        if requested."""
+        cmd_args = command.expand()
+        cmd_txt = str(command)
+        if (len(cmd_args) == 0) or (start and len(cmd_args[0]) == 0):
+            raise JobCreationException("No program to execute")
+        cmd_dtvs = command.expand_directives()
+        replace_id = None 
+        for dtv in cmd_dtvs:
+            if dtv.startswith("replace"):
+                cmd_txt = re.sub("#replace[0-9]* *", "", cmd_txt)
+                try:
+                    replace_id = int(dtv[7:])
+                except ValueError:
+                    raise JobCreationException("Badly formatted #replace")
+        if start and replace_id is not None:
+            replace_job = self.get_job(replace_id)
+            if replace_job is None:
+                raise JobCreationException("No job to replace")
+            if replace_job.get_state() == "Running":
+                raise JobCreationException("Cannot replace a running job")
+
+            # Don't create a new class in the case of replace, because there
+            # may be references to it elsewhere, just repurpose the existing
+            # one.  This means it can't change from External to BuiltIn or from
+            # one BuiltIn to another.
+            job = replace_job
+            job._set_command(cmd_args, cmd_txt, cmd_dtvs, "Initializing")
+
         else:
-            jobcls = ExternalJob
+            if cmd_args[0] in loaded_builtins:
+                jobcls = loaded_builtins[cmd_args[0]]
+            else:
+                jobcls = ExternalJob
 
-        job = jobcls.create_job(cmd, cmdline, directives, state)
+            job = jobcls.create_job(cmd_args, cmd_txt, cmd_dtvs,
+                                    start and "Initializing" or "NotRun")
+
+            # Extend the all_jobs array to include the newly created job.
+            for i in range(len(self.all_jobs), job.jid):
+                self.all_jobs.append(os.path.join(config.jobs_dir, str(i)))
+                self.get_job(i)
+            self.all_jobs.append(job)
+            for listener in self.listeners:
+                listener.on_add_job(job)
+
+        if start:
+            job.start()
 
-        # Extend the all_jobs array to include the newly created job.
-        for i in range(len(self.all_jobs), job.jid):
-            self.all_jobs.append(os.path.join(config.jobs_dir, str(i)))
-            self.get_job(i)
-        self.all_jobs.append(job)
-        for listener in self.listeners:
-            listener.on_add_job(job)
         return job
 
     def load_job(self, datadir):
@@ -259,8 +310,7 @@ class JobOutputThread(threading.Thread):
 
     def output_normal(self, text):
         """Handle Job output in normal mode."""
-        nr = self.job.output.append_region(text)
-        nr.source = self.channel
+        self.job.output.append_region(text, {"source" : self.channel})
         self.outfile.write(text)
         self.outfile.flush()
         for listener in self.job.listeners:
@@ -357,15 +407,11 @@ class Job(object):
     def create_job(cls, cmd, cmdline, directives, state="NotRun"):
         """Return a new job object created from the given command."""
         job = cls()
-        job.cmd = cmd
-        job.cmdline = cmdline
-        job.directives_in = directives
-        job._set_directives()
-        job.cwd = os.getcwd()
+        job._set_command(cmd, cmdline, directives, state)
+        job.cwd = system.get_cwd()
         job.username = system.get_username()
         job.hostname = system.get_hostname()
         job.is_new = True
-        job.state = state
 
         # All jobs need an id, so get one for this job.  This is done by
         # finding the job directory with the highest number and adding one.
@@ -410,7 +456,7 @@ class Job(object):
     def __init__(self):
         self.cmd = None
         self.cmdline = None
-        self.output = content.TextContent()
+        self.output = content.text.MutableText()
         self.cwd = None
         self.username = None
         self.hostname = None
@@ -427,13 +473,28 @@ class Job(object):
         self.wants_terminal = False
         self.has_terminal = False
 
+    def _set_command(self, cmd_args, cmdline, directives, state):
+        """Set various members for the given command info."""
+        self.cmd = cmd_args
+        self.cmdline = cmdline
+        self.state = state
+        self.directives_in = directives
+        self._set_directives()
+
+        self.output[0:] = []
+        self.ps = None
+        self.ps_code = None
+        self.pid = None
+        self.writeable = False
+        self.wants_terminal = False
+
     def _set_directives(self):
         """When directives_in and directives_default are set, use this to
         set directives."""
         if not hasattr(self,"directives_default"):
             self.directives_default = []
         dlist = self.directives_default + self.directives_in
-        self.directives = command.HashDirective.canonify(dlist)
+        self.directives = canonify_directives(dlist)
 
     def start(self):
         if self.get_state() == "Running":
@@ -535,7 +596,6 @@ class ExternalJob(Job):
             # An exclusive job is still remembered like other jobs, except its
             # output and input are not recorded.  It is executed in the main
             # thread, since not much else can go on while it's running.
-            self.wants_terminal = True
             self.do_exclusive()
             return
 
@@ -584,13 +644,14 @@ class ExternalJob(Job):
         job terminates, or its process is stopped.  The job will not be
         permitted to run until the display thread grants control over the
         terminal.  If start_proc is True, then the job process will be started,
-        otherwise it will be resumed with a CONT signal.
+        otherwise it will be resumed with a CONT signal.  A request will be
+        sent to display this job before waiting for the terminal.  """
 
-        It's important that the job be correctly set up with the display, so
-        that it will be granted terminal access."""
-        if not self.wants_terminal:
-            logging.warn("do_exclusive(): self.wants_terminal must be true")
-            return
+        # Turn off wants_terminal briefly while displaying the job, so that we
+        # don't get caught in a recusrive call to do_exclusive.
+        self.wants_terminal = False
+        hsh.display_obj.display_job(self, show=True, focus=True)
+        self.wants_terminal = True
 
         while not self.has_terminal:
             time.sleep(0.1)
@@ -670,6 +731,9 @@ class ExternalJob(Job):
         communicates solely by sending a STOP signal to the job.  The main
         thread takes responsibility for noticing that the job has stopped and
         changing object state."""
+        if self.ps is None:
+            # When the job is just starting or ending ps might not exist.
+            return
         if self.ps.pid and (self.get_state() == "Running"):
             try:
                 os.kill(self.ps.pid, signal.SIGSTOP)
@@ -715,8 +779,8 @@ class BuiltInJob(Job):
         super(BuiltInJob, self).start()
 
         try:
-            self.do_start()
             self.state = "Succeeded"
+            self.do_start()
             self._write_job_state()
             for listener in self.listeners:
                 listener.on_job_output(self)
