@@ -23,9 +23,9 @@ message = """
                           Welcome to hsh
 
 hsh  Copyright (C) 2010  Alexander Taler <dissent@0--0.org>
-This program comes with ABSOLUTELY NO WARRANTY; type `+copying' for details.
+This program comes with ABSOLUTELY NO WARRANTY; type `copying' for details.
 This is free software, and you are welcome to redistribute it
-under certain conditions; type `+copying' for details.
+under certain conditions; type `copying' for details.
 
 (C- means use the control key, M- means use the Meta/Alt/Esc key)
 
@@ -38,6 +38,7 @@ M-n to display next job
 M-r to run the currently displayed job again
 M-i to change input mode for the current job
 M-c to close standard input for the current job
+M-h to hide output of the current job
 
 C-k to cut text to end-of-line
 C-y to paste text
@@ -45,7 +46,7 @@ C-y to paste text
 Function keys select main view
 F1  - help
 F2  - job output
-F3  - job list
+F3  - session list
 F4  - running jobs
 F5  - command history
 F6  - job history
